/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.auth

import android.app.Activity
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.os.Bundle
import android.os.Handler
import android.os.Looper
import android.os.ResultReceiver
import com.kakao.sdk.common.model.Description
import com.kakao.sdk.common.model.KakaoSdkError
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.partner.auth.model.AgeAuthError
import com.kakao.sdk.partner.auth.model.AgeAuthErrorCause
import com.kakao.sdk.partner.auth.model.AgeAuthLevel
import io.reactivex.*

/**
 * @suppress
 */
class RxAgeAuthApiManager {

    fun verifyAgeWithAuthentication (
        context: Context,
        ageLimit : Int? = null,
        authLevel : AgeAuthLevel? = null,
        skipTerms : Boolean? = null,
        adultsOnly : Boolean? = null,
        underAge : Boolean? = false) : Single<Unit> = Single.create<Unit>{
            val uri = AgeAuthManager.instance.makeUri(ageLimit, authLevel, skipTerms, adultsOnly, underAge)

            SdkLog.i(uri)
            try {
                val intent = Intent(context, AgeAuthActivity::class.java)
                    .putExtra(com.kakao.sdk.auth.Constants.KEY_BUNDLE, Bundle().apply {
                        putParcelable(com.kakao.sdk.auth.Constants.KEY_RESULT_RECEIVER, resultReceiver(it))
                        putParcelable(com.kakao.sdk.auth.Constants.KEY_FULL_URI, uri)
                    }).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)

                context.startActivity(intent)
            } catch (startActivityError: Throwable) {
                it.onError(startActivityError)
            }
        }.compose(handleVerifyAgeError())


    @JvmSynthetic
    internal fun resultReceiver(
        emitter: SingleEmitter<Unit>
    ): ResultReceiver =
        object : ResultReceiver(Handler(Looper.getMainLooper())) {
            override fun onReceiveResult(resultCode: Int, resultData: Bundle?) {
                onReceivedResult(resultCode, resultData, emitter)
            }
        }

    @JvmSynthetic
    internal fun onReceivedResult(
        resultCode: Int,
        resultData: Bundle?,
        emitter: SingleEmitter<Unit>
    ) {
        if (emitter.isDisposed) return

        when (resultCode) {
            Activity.RESULT_OK -> {
                val uri = resultData?.getParcelable<Uri>(com.kakao.sdk.auth.Constants.KEY_URL)
                var param = 0
                try {
                    param = uri?.getQueryParameter(Constants.STATUS)?.toInt() ?: -1
                } catch (e : Exception) {
                    param = -9999
                    SdkLog.e(e)
                }

                if (param == 0) {
                    emitter.onSuccess(Unit)
                    return
                }

                val cause = AgeAuthErrorCause.values().find { it.status == param }
                if (cause == null) {
                    emitter.onError(AgeAuthError(AgeAuthErrorCause.UNKNOWN, "unknown error."))
                    return
                }

                val description = cause.javaClass.getField(cause.name)?.getAnnotation(Description::class.java)?.value ?: "unknown error."
                SdkLog.i("status ${cause.status} , description ${description}")
                emitter.onError(AgeAuthError(cause, description))
            }
            Activity.RESULT_CANCELED -> {
                var error = resultData?.getSerializable(com.kakao.sdk.auth.Constants.KEY_EXCEPTION) as KakaoSdkError
                emitter.onError(error)
            }
            else -> throw IllegalArgumentException("Unknown resultCode in RxAuthApiClient#onReceivedResult()")
        }
    }

    @JvmSynthetic
    internal fun  handleVerifyAgeError(): SingleTransformer<Unit, Unit> = SingleTransformer{
        it.doOnError { SdkLog.e(it) }
            .doOnSuccess {  }
    }


    companion object {
        /**
         * 간편한 API 호출을 위해 기본 제공되는 singleton 객체
         */
        @JvmStatic
        val instance by lazy { RxAgeAuthApiManager() }
    }
}