/*
  Copyright 2021 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.partner.friend.model

import android.os.Parcelable
import com.google.gson.annotations.SerializedName
import com.kakao.sdk.partner.friend.Constants
import kotlinx.android.parcel.Parcelize

/**
 * @property title 친구 선택 화면의 이름
 * @property pickerFriendOrder 친구 목록 정렬 기준
 * @property pickerFriendType 친구를 가져올 서비스
 * @property pickerFriendFilter 친구 목록 필터링 기준
 * @property viewAppearance 친구 선택 화면 모드
 * @property countryCodes 카카오톡에 등록된 전화번호의 국가 코드를 토대로 요청 결과를 필터링할 때 사용
 * @property usingOsFilter 카카오톡 가입 기기의 OS 정보를 토대로 요청 결과를 필터링할 때 사용
 * @property orientation 친구 선택 화면의 방향
 * @property enableSearch 친구 검색 기능 사용 여부
 * @property searchPlaceHolderText 친구 검색창에 표시될 텍스트
 * @property enableIndex 친구 목록 인덱스뷰 사용 여부
 * @property showMyProfile 내 프로필 표시 여부
 * @property showFavorite 즐겨찾기 친구 표시 여부
 * @property disableSelectReasons 친구 선택 불가 사유
 * @property maxPickableCount 선택 가능한 친구 수의 최대값
 * @property minPickableCount 선택 가능한 친구 수의 최소값
 * @property showPickedFriend 선택한 친구 표시 여부 (멀티 피커에만 사용 가능)
 * @property displayAllProfile 서비스 미가입자의 프로필 정보 표시 여부
 */
@Parcelize
class PickerRequestParams(
    val title: String? = null,
    val pickerFriendOrder: PickerFriendOrder? = null,
    val pickerFriendType: PickerFriendType? = Constants.DEFAULT_PICKER_FRIEND_TYPE,
    val pickerFriendFilter: PickerFriendFilter? = Constants.DEFAULT_PICKER_FRIEND_FILTER,
    val viewAppearance: ViewAppearance? = Constants.DEFAULT_PICKER_VIEW_APPEARANCE,
    val countryCodes: List<String>? = null,
    val usingOsFilter: PickerUsingOsFilter? = Constants.DEFAULT_OS_FILTER,
    val orientation: PickerOrientation? = Constants.DEFAULT_PICKER_ORIENTATION,
    val enableSearch: Boolean? = Constants.DEFAULT_ENABLE_SEARCH,
    val searchPlaceHolderText: String? = null,
    val enableIndex: Boolean? = Constants.DEFAULT_ENABLE_INDEX,
    val showMyProfile: Boolean? = Constants.DEFAULT_SHOW_MY_PROFILE,
    val showFavorite: Boolean? = Constants.DEFAULT_SHOW_FAVORITE,
    val disableSelectReasons: DisableSelectReason? = null,
    val maxPickableCount: Int? = Constants.DEFAULT_MAX_PICKABLE_COUNT,
    val minPickableCount: Int? = Constants.DEFAULT_MIN_PICKABLE_COUNT,
    val showPickedFriend: Boolean? = Constants.DEFAULT_SHOW_PICKED_FRIENDS,
    val displayAllProfile: Boolean? = false
) : Parcelable

/**
 * 친구 목록 정렬 기준
 */
enum class PickerFriendOrder {
    /**
     * 나이 AGE 선택 시, 프로필 생년월일 기준으로 만 19세 이상의 사용자들이 우선 정렬되고, 미성년자 그룹은 하단에 정렬(각 성인/미성년자 그룹 내에서는 이름 순으로 정렬). 이 때, 친구 목록 인덱스 뷰는 비활성화 처리됨
     */
    @SerializedName("age")
    AGE
}

/**
 * 친구를 가져올 서비스
 */
enum class PickerFriendType {
    /**
     * 카카오톡 친구만 조회
     */
    @SerializedName("talk")
    TALK,

    /**
     * 카카오스토리 친구만 조회
     */
    @SerializedName("story")
    STORY,

    /**
     * 카카오톡 친구와 카카오스토리 친구를 모두 조회
     */
    @SerializedName("talkstory")
    TALKSTORY
}

/**
 * 친구 목록 필터링 기준
 */
enum class PickerFriendFilter {
    /**
     * 필터링하지 않음
     */
    @SerializedName("none")
    NONE,

    /**
     * 앱에 연결된 친구만 조회
     */
    @SerializedName("registered")
    REGISTERED,

    /**
     * 앱과 연결되지 않아 초대 가능한 친구만 조회
     */
    @SerializedName("invitable")
    INVITABLE
}

/**
 * 친구 선택 화면 모드
 */
enum class ViewAppearance {
    /**
     * 라이트 모드
     */
    LIGHT,

    /**
     * 다크 모드
     */
    DARK,

    /**
     * 시스템 디스플레이 설정에 따라 라이트 또는 다크 모드로 자동 전환
     */
    AUTO
}

/**
 * 친구의 카카오톡 가입 기기의 OS 정보를 토대로 요청 결과를 필터링할 때 사용
 */
enum class PickerUsingOsFilter {
    /**
     * Android와 iOS 사용자 모두 조회
     */
    ALL,

    /**
     * Android 사용자만 조회
     */
    ANDROID,

    /**
     * iOS 사용자만 조회
     */
    IOS
}

/**
 * 친구 선택 화면의 방향
 */
enum class PickerOrientation {
    /**
     * 세로 모드
     */
    PORTRAIT,

    /**
     * 가로 모드
     */
    LANDSCAPE,

    /**
     * 시스템 설정에 따라 세로 또는 가로 모드로 자동 전환
     */
    AUTO
}

/**
 * 친구 선택 불가 사유
 */
enum class DisableSelectReason {
    /**
     * 메시지 수신 거부로 인해 친구 선택 불가
     */
    MSG_BLOCKED
}